// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2014 - Scilab Enterprises - Bruno Jofret
// Copyright (C) 2014 - Scilab Enterprises - Cedric DELAMARRE
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt


abs_path = get_absolute_file_path("shallow_gpu.sce");
rand("seed", 0);
// grid size
s=150;
// refresh ?
refresh = 50;
// rain delay
rainDelay = 2;

level=zeros(s,s);
vx=zeros(s,s);
vy=zeros(s,s);

figure1 = gcf();

sh = 1:s;
h=zeros(sh);
for i=sh
    for j=sh
        h(i,j) = 1 / (1 + (i - (s / 2))^2 + (j - (s / 2))^2)^2;
    end
end
h=h/sum(h, '*');
szh = size(h);
i=-1;
nd=50;

plot3d(zeros(2*s-1, 2*s-1));
f = gcf();
c = (37:100)';
cyan = [0*c, c, c]/100;
f.color_map = cyan;
a = gca();

a.axes_visible = "off";
a.x_label.visible = "off";
a.y_label.visible = "off";
a.z_label.visible = "off";
a.rotation_angles = [89.75, 60];
a.data_bounds = [0, 0, -0.3 ; 2*s, 2*s, 0.6];
e = gce();
e.color_mode = -4;
e.color_flag = 1;
xtitle("shallow water flow");
tic();

conv2_1 = [1, 2, 1 ; 2, 10488, 2; 1, 2, 1] ./ 10500;
conv2_2 = [1, 5, 1 ; 5, 6, 5 ; 1, 5, 1] ./ 30;

d_conv2_1 = gpuSetData(conv2_1);
d_conv2_2 = gpuSetData(conv2_2);
[rows, cols] = size(conv2_1);
edgM = (rows - 1) / 2;
edgN = (cols - 1) / 2;

nbMaxThread = 16;
if s < nbMaxThread then
    nb_threadX = s;
else
    nb_threadX = nbMaxThread;
end

if s < nbMaxThread then
    nb_threadY = s;
else
    nb_threadY = nbMaxThread;
end
nb_blockY = ceil(s / nb_threadX);
nb_blockX = ceil(s / nb_threadY);

bin       = gpuBuild(abs_path + "shallow_kernels");
cpt_conv  = gpuLoadFunction(bin, "compute_conv", nb_threadY, nb_threadX, nb_blockY, nb_blockX);
cpt_copy  = gpuLoadFunction(bin, "compute_copy", nb_threadY, nb_threadX, nb_blockY, nb_blockX);
cpt_vxvy  = gpuLoadFunction(bin, "compute_vxvy", nb_threadY, nb_threadX, nb_blockY, nb_blockX);
cpt_level = gpuLoadFunction(bin, "compute_level", nb_threadY, nb_threadX, nb_blockY, nb_blockX);

d_level = gpuSetData(level);
d_vx    = gpuSetData(vx);
d_vy    = gpuSetData(vy);
d_Out   = gpuSetData(level);
d_h     = gpuSetData(h);

for k=1:10000
    i = i + 1;
    if(i == 0 | i > nd)
        nd = floor(nd + rand() * 2000 + rainDelay);
        px = floor(rand() * s)+1;
        py = floor(rand() * s)+1;

        level = gpuGetData(d_level);
        level = level + (rand() * 20 - 10) .* h([px:szh(1), 1:px-1],[py:szh(2) 1:py-1]);

        clear d_level;
        d_level = gpuSetData(level);
    end

    cpt_vxvy(d_level, d_vx, d_vy, int32(s), int32(s));
    cpt_level(d_level, d_vx, d_vy, int32(s), int32(s));
    cpt_conv(d_level, int32(s), int32(s), d_conv2_1, int32(rows), int32(cols), d_Out, int32(s), int32(s), int32(edgM), int32(edgN));
    cpt_copy(d_Out, d_level, int32(s), int32(s));

    if(modulo(i, refresh) == 0)
        [XI,YI] = meshgrid(1:0.5:s, 1:0.5:s);

        cpt_conv(d_level, int32(s), int32(s), d_conv2_2, int32(rows), int32(cols), d_Out, int32(s), int32(s), int32(edgM), int32(edgN));
        spl=gpuGetData(d_Out);

        C = splin2d(1:s, 1:s, spl);
        spl=interp2d(XI, YI, 1:s, 1:s, C);
        e.data.z = spl;
    end
end
toc()
